from lib import trace_loop

SERVICE_NAME = "my-otel-test"
NUM_ADDS = 12

if __name__ == "__main__":
    trace_loop(NUM_ADDS)

class MyOtelTest:
    def requirements(self):
        return "opentelemetry-distro", "opentelemetry-exporter-otlp-proto-grpc"

    def environment_variables(self):
        return {
            "OTEL_SERVICE_NAME": SERVICE_NAME,
            "OTEL_EXPERIMENTAL_RESOURCE_DETECTORS": "process",
        }

    def wrapper_command(self):
        return "opentelemetry-instrument"

    def on_start(self):
        return None

    def on_stop(self, telemetry, stdout: str, stderr: str, returncode: int) -> None:
        print(f"returncode: [{returncode}]")

    def is_http(self):
        return False



from lib import trace_loop

# WHL = "/Users/pcollins/github/signalfx/splunk-otel-python/dist/splunk_opentelemetry-0.0.1-py3-none-any.whl"
# WHL = "/Users/pcollins/github/signalfx/sop-worktree/dist/splunk_opentelemetry-1.20.0.dev0-py3-none-any.whl[otlp]"
# WHL = "/Users/pcollins/github/signalfx/splunk-otel-python/dist/splunk_opentelemetry-0.0.1-py3-none-any.whl[otlp]"
# WHL = "/Users/pcollins/github/signalfx/sop-worktree/dist/splunk_opentelemetry-1.20.0-py3-none-any.whl[otlp]"
# WHL = "/Users/pcollins/github/signalfx/sop-worktree/dist/splunk_opentelemetry-1.21.0-py3-none-any.whl[all]"
# WHL = "/Users/pcollins/github/signalfx/sop-v2/dist/splunk_opentelemetry-2.0.0a1-py3-none-any.whl"
# WHL = "/Users/pcollins/github/signalfx/splunk-otel-python/dist/splunk_opentelemetry-0.0.1-py3-none-any.whl"
WHL = "/Users/pcollins/github/signalfx/sop-v2-prs/dist/splunk_opentelemetry-2.0.0a1-py3-none-any.whl"

SERVICE_NAME = "my-otel-test"
NUM_ADDS = 12

if __name__ == "__main__":
    trace_loop(NUM_ADDS)


class MyOtelTest:
    def requirements(self):
        return (WHL,)

    def environment_variables(self):
        return {
            "OTEL_SERVICE_NAME": SERVICE_NAME,
        }

    def wrapper_command(self):
        return "opentelemetry-instrument"

    def on_start(self):
        return None

    def on_stop(self, telemetry, stdout: str, stderr: str, returncode: int) -> None:
        print(f"script completed with return code {returncode}")

    def is_http(self):
        return False



import dataclasses
import json
from typing import List, Optional, Union

from google.protobuf.json_format import MessageToDict
from opentelemetry.proto.collector.logs.v1.logs_service_pb2 import (
    ExportLogsServiceRequest,
)
from opentelemetry.proto.collector.metrics.v1.metrics_service_pb2 import (
    ExportMetricsServiceRequest,
)
from opentelemetry.proto.collector.trace.v1.trace_service_pb2 import (
    ExportTraceServiceRequest,
)


@dataclasses.dataclass
class Request:
    """
    Wraps a grpc message (metric, trace, or log), http headers that came in with the message, and the time elapsed
    between the start of the test and the receipt of the message.
    """

    pbreq: Union[
        ExportTraceServiceRequest, ExportMetricsServiceRequest, ExportLogsServiceRequest
    ]
    headers: dict
    test_elapsed_ms: int

    def get_header(self, name):
        return self.headers.get(name)

    def to_json(self):
        return json.dumps(self.to_dict())

    def to_dict(self):
        return {
            "pbreq": MessageToDict(self.pbreq),
            "headers": self.headers,
            "test_elapsed_ms": self.test_elapsed_ms,
        }


class Telemetry:
    """
    Wraps lists of metric, trace, and log requests sent during a single oteltest script run. An instance is passed in to
    OtelTest#on_stop().
    """

    def __init__(
        self,
        metric_requests: Optional[List[Request]] = None,
        trace_requests: Optional[List[Request]] = None,
        log_requests: Optional[List[Request]] = None,
    ):
        self.metric_requests: List[Request] = metric_requests or []
        self.trace_requests: List[Request] = trace_requests or []
        self.log_requests: List[Request] = log_requests or []

    def add_metric(
        self, pbreq: ExportMetricsServiceRequest, headers: dict, test_elapsed_ms: int
    ):
        self.metric_requests.append(Request(pbreq, headers, test_elapsed_ms))

    def add_trace(
        self, pbreq: ExportTraceServiceRequest, headers: dict, test_elapsed_ms: int
    ):
        self.trace_requests.append(Request(pbreq, headers, test_elapsed_ms))

    def add_log(
        self, pbreq: ExportLogsServiceRequest, headers: dict, test_elapsed_ms: int
    ):
        self.log_requests.append(Request(pbreq, headers, test_elapsed_ms))

    def get_metric_requests(self) -> List[Request]:
        return self.metric_requests

    def get_trace_requests(self) -> List[Request]:
        return self.trace_requests

    def get_logs_requests(self) -> List[Request]:
        return self.log_requests

    def __str__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps(self.to_dict(), indent=2)

    def to_dict(self):
        return {
            "metric_requests": [req.to_dict() for req in self.metric_requests],
            "trace_requests": [req.to_dict() for req in self.trace_requests],
            "log_requests": [req.to_dict() for req in self.log_requests],
        }


_metrics_path = ["metric_requests", "resource_metrics", "scope_metrics", "metrics"]
_span_path = ["trace_requests", "resource_spans", "scope_spans", "spans"]
_logs_path = ["log_requests", "resource_logs", "scope_logs", "log_records"]


def has_log_attribute(tel, key) -> bool:
    for log in get_logs(tel):
        for attribute in log.attributes:
            if attribute.key == key:
                return True
    return False


def count_metrics(telemetry) -> int:
    return len(_get_leaves(telemetry, *_metrics_path))


def get_metric_names(telemetry) -> set:
    return {leaf.name for leaf in _get_leaves(telemetry, *_metrics_path)}


def count_spans(telemetry) -> int:
    return len(get_spans(telemetry))


def count_logs(telemetry) -> int:
    return len(get_logs(telemetry))


def span_names(telemetry) -> set:
    return {leaf.name for leaf in _get_leaves(telemetry, *_span_path)}


def get_metrics(telemetry):
    return _get_leaves(telemetry, *_metrics_path)


def get_spans(telemetry):
    return _get_leaves(telemetry, *_span_path)


def get_logs(telemetry):
    return _get_leaves(telemetry, *_logs_path)


def _get_leaves(telemetry, k1, k2, k3, k4):
    out = []
    for a1 in getattr(telemetry, k1):
        for a2 in getattr(a1.pbreq, k2):
            for a3 in getattr(a2, k3):
                for a4 in getattr(a3, k4):
                    out.append(a4)
    return out


def has_trace_header(telemetry, key, expected) -> bool:
    for req in telemetry.trace_requests:
        actual = req.get_header(key)
        if expected == actual:
            return True
    return False


def first_span(tel: Telemetry):
    return span_at_index(tel, 0, 0, 0, 0)


def span_at_index(tel: Telemetry, i: int, j: int, k: int, l: int):
    if len(tel.trace_requests):
        req = tel.trace_requests[i]
        if len(req.pbreq.resource_spans):
            rs = req.pbreq.resource_spans[j]
            if len(rs.scope_spans):
                ss = rs.scope_spans[k]
                if len(ss.spans):
                    return ss.spans[l]
    return None


def span_attribute_by_name(span, attr_name) -> Optional[str]:
    for attr in span.attributes:
        if attr.key == attr_name:
            if attr.value.HasField("string_value"):
                return attr.value.string_value
    return None



import glob
import importlib
import importlib.util
import inspect
import os
import shutil
import subprocess
import sys
import tempfile
import typing
import venv
from logging import Logger
from pathlib import Path

from oteltest import OtelTest
from oteltest.sink import GrpcSink, HttpSink, raise_if_port_in_use
from oteltest.sink.handler import AccumulatingHandler
from oteltest.version import __version__


def run(script_paths: [str], venv_parent_dir: str, json_dir: str, logger: Logger):
    logger.info("oteltest version %s", __version__)

    temp_dir = venv_parent_dir or tempfile.mkdtemp()
    logger.info("Using temp dir for venvs: %s", temp_dir)

    for script_path in script_paths:
        if os.path.isdir(script_path):
            handle_dir(script_path, temp_dir, json_dir, logger)
        elif os.path.isfile(script_path):
            handle_file(script_path, temp_dir, json_dir, logger)
        else:
            logger.warning("argument %s does not exist", script_path)


def handle_dir(dir_path, temp_dir, json_dir, logger):
    sys.path.append(dir_path)
    for script in ls_scripts(dir_path):
        logger.info("Setting up environment for script %s", script)
        setup_script_environment(temp_dir, dir_path, script, json_dir, logger)


def handle_file(file_path, temp_dir, json_dir, logger):
    logger.info("Setting up environment for file %s", file_path)
    script_dir = os.path.dirname(file_path)
    sys.path.append(script_dir)
    setup_script_environment(
        temp_dir, script_dir, os.path.basename(file_path), json_dir, logger
    )


def ls_scripts(script_dir):
    original_dir = os.getcwd()
    os.chdir(script_dir)
    scripts = glob.glob("*.py")
    os.chdir(original_dir)
    return scripts


def setup_script_environment(
    venv_parent: str, script_dir: str, script: str, json_dir_base: str, logger: Logger
):
    module_name = script[:-3]
    module_path = os.path.join(script_dir, script)
    oteltest_class = load_oteltest_class_for_script(module_name, module_path, logger)
    if oteltest_class is None:
        logger.info("No oteltest class present in [%s]", module_name)
        return
    oteltest_instance = oteltest_class()

    handler = AccumulatingHandler()
    if hasattr(oteltest_instance, "is_http") and oteltest_instance.is_http():
        raise_if_port_in_use(4318)
        sink = HttpSink(handler, logger)
    else:
        raise_if_port_in_use(4317)
        sink = GrpcSink(handler, logger)
    sink.start()

    script_venv = Venv(str(Path(venv_parent) / module_name), logger)
    script_venv.create()

    pip_path = script_venv.path_to_executable("pip")

    for req in oteltest_instance.requirements():
        logger.info("Will install requirement: '%s'", req)
        run_subprocess([pip_path, "install", req], logger)

    stdout, stderr, returncode = run_python_script(
        start_subprocess, script_dir, script, oteltest_instance, script_venv, logger
    )
    print_subprocess_result(stdout, stderr, returncode, logger)

    json_dir = os.path.join(script_dir, json_dir_base)
    filename = get_next_json_file(json_dir, module_name)
    logger.info("Will save telemetry to %s", filename)
    save_telemetry_json(json_dir, filename, handler.telemetry_to_json())

    oteltest_instance.on_stop(handler.telemetry, stdout, stderr, returncode)
    logger.info("PASSED: %s", script)
    sink.stop()


def get_next_json_file(path_str: str, module_name: str):
    path = Path(path_str)
    path.mkdir(exist_ok=True)
    max_index = -1
    for file in path.glob(f"{module_name}.*.json"):
        last_part = file.stem.split(".")[-1]
        if last_part.isdigit():
            index = int(last_part)
            if index > max_index:
                max_index = index
    return f"{module_name}.{max_index + 1}.json"


def save_telemetry_json(script_dir: str, file_name: str, json_str: str):
    path = Path(script_dir) / file_name
    with open(str(path), "w", encoding="utf-8") as file:
        file.write(json_str)


def run_python_script(
    start_subprocess_func,
    script_dir: str,
    script: str,
    oteltest_instance,
    script_venv,
    logger: Logger,
) -> typing.Tuple[str, str, int]:
    logger.info("Running python script: %s", script)
    python_script_cmd = [
        script_venv.path_to_executable("python"),
        str(Path(script_dir) / script),
    ]

    wrapper_script = oteltest_instance.wrapper_command()
    if wrapper_script:
        python_script_cmd.insert(0, script_venv.path_to_executable(wrapper_script))

    # typically python_script_cmd will be ["opentelemetry-instrument", "python", "foo.py"] but with full paths
    logger.info(f"Start subprocess: %s", python_script_cmd)
    proc = start_subprocess_func(
        python_script_cmd, oteltest_instance.environment_variables()
    )
    timeout_seconds = oteltest_instance.on_start()
    if timeout_seconds is None:
        logger.info("Will wait for %s to finish by itself", script)
    else:
        logger.info(
            "Will wait for %d seconds for %s to finish", timeout_seconds, script
        )
    try:
        stdout, stderr = proc.communicate(timeout=timeout_seconds)
        return stdout, stderr, proc.returncode
    except subprocess.TimeoutExpired as ex:
        proc.kill()
        logger.info("Script %s terminated", script)
        return decode(ex.stdout), decode(ex.stderr), proc.returncode


def start_subprocess(python_script_cmd, env):
    return subprocess.Popen(
        python_script_cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
        env=env,
    )


def decode(b: typing.Optional[bytes]) -> str:
    return b.decode("utf-8") if b else ""


def run_subprocess(args, logger: Logger):
    logger.info("Subprocess: %s", args)
    result = subprocess.run(
        args,
        capture_output=True,
        text=True,
        check=True,
    )
    print_subprocess_result(result.stdout, result.stderr, result.returncode, logger)


def print_subprocess_result(stdout: str, stderr: str, returncode: int, logger: Logger):
    logger.info("Return Code: [%s]", returncode)  # %s because can be None
    logger.info("Standard Output:")
    if stdout:
        print(stdout)
    logger.info("Standard Error:")
    if stderr:
        print(stderr)
    logger.info("End Subprocess")


def load_oteltest_class_for_script(module_name, module_path, logger: Logger):
    logger.debug(
        "loading spec from file: module_name [%s] module_path [%s]",
        module_name,
        module_path,
    )
    spec = importlib.util.spec_from_file_location(module_name, module_path)
    logger.debug("spec loaded: [%s]", spec)
    module = importlib.util.module_from_spec(spec)
    logger.debug("module loaded: [%s]", module)
    spec.loader.exec_module(module)
    for attr_name in dir(module):
        value = getattr(module, attr_name)
        if is_test_class(value):
            logger.debug("found test class: [%s]", value)
            return value
    return None


def is_test_class(value):
    return inspect.isclass(value) and (
        is_strict_subclass(value) or "OtelTest" in value.__name__
    )


def is_strict_subclass(value):
    return (
        issubclass(value, OtelTest)
        and value is not OtelTest
        and not inspect.isabstract(value)
    )


class Venv:
    def __init__(self, venv_dir, logger: Logger):
        self.venv_dir = venv_dir
        self.logger = logger

    def create(self):
        if os.path.exists(self.venv_dir):
            self.logger.info(
                "Path to virtual env [%s] already exists, skipping creation",
                self.venv_dir,
            )
        else:
            venv.create(self.venv_dir, with_pip=True)

    def path_to_executable(self, executable_name: str):
        return f"{self.venv_dir}/bin/{executable_name}"

    def rm(self):
        shutil.rmtree(self.venv_dir)



import time

from opentelemetry import trace

def trace_loop(loops):
    tracer = trace.get_tracer("my-tracer")
    for i in range(loops):
        with tracer.start_as_current_span("my-span"):
            print(f"loop {i + 1}/{loops}")
            time.sleep(0.5)




from typing import Mapping, Optional, Sequence

from oteltest import OtelTest, Telemetry

PORT = 8888

HOST = "127.0.0.1"


def main():
    from flask import Flask

    app = Flask(__name__)

    @app.route("/")
    def home():
        return "hello"

    app.run(host=HOST, port=PORT)


if __name__ == "__main__":
    main()


class FixProblemOTT(OtelTest):
    def environment_variables(self) -> Mapping[str, str]:
        return {
            "OTEL_SERVICE_NAME": "my-svc",
        }

    def requirements(self) -> Sequence[str]:
        return [
            "splunk-opentelemetry[all]",
            "oteltest",
            "flask",
            "opentelemetry-instrumentation-flask==0.48b0",
        ]

    def wrapper_command(self) -> str:
        return "opentelemetry-instrument"

    def on_start(self) -> Optional[float]:  # noqa: FA100
        import http.client
        import time

        time.sleep(6)

        conn = http.client.HTTPConnection(HOST, PORT)
        conn.request("GET", "/")

        response = conn.getresponse()
        assert_server_timing_headers_found(response)

        conn.close()

        return 6

    def on_stop(self, tel: Telemetry, stdout: str, stderr: str, returncode: int) -> None:
        pass

    def is_http(self) -> bool:
        pass


def assert_server_timing_headers_found(response):
    # Server-Timing: traceparent;desc="00-e899d68fca52b66d3facae0bdaf764db-159efb97d6b56568-01"
    # Access-Control-Expose-Headers: Server-Timing
    server_timing_header_found = False
    access_control_header_found = False
    for header, _ in response.getheaders():
        if header == "Server-Timing":
            server_timing_header_found = True
        elif header == "Access-Control-Expose-Headers":
            access_control_header_found = True
    assert server_timing_header_found
    assert access_control_header_found



from lib import trace_loop

SERVICE_NAME = "my-otel-test"
NUM_ADDS = 12


if __name__ == "__main__":
    trace_loop(NUM_ADDS)


# Since we're not inheriting from the OtelTest base class (to avoid depending on it) we make sure our class name
# contains "OtelTest".
class MyOtelTest:
    def requirements(self):
        return (
            "opentelemetry-distro",
            "opentelemetry-instrumentation-system-metrics",
            "opentelemetry-exporter-otlp-proto-grpc",
        )

    def environment_variables(self):
        return {
            "OTEL_SERVICE_NAME": SERVICE_NAME,
        }

    def wrapper_command(self):
        return "opentelemetry-instrument"

    def on_start(self):
        pass

    def on_stop(self, telemetry, stdout: str, stderr: str, returncode: int) -> None:
        pass

    def is_http(self):
        return False



import argparse
import logging

from oteltest.private import run
from oteltest.version import __version__ as version

def main():
    parser = argparse.ArgumentParser(description=f"Version {version}")

    d_help = "An optional override directory to hold per-script venv directories."
    parser.add_argument(
        "-d", "--venv-parent-dir", type=str, required=False, help=d_help
    )

    j_help = "An optional value to hold the directory into which json telemetry files are written"
    parser.add_argument(
        "-j", "--json-dir", type=str, required=False, help=j_help, default="json"
    )

    v_help = "Enable verbose output; include this flag for detailed logging."
    parser.add_argument("-v", "--verbose", action="store_true", help=v_help)

    parser.add_argument(
        "script_paths",
        nargs="+",
        help="One or more oteltest files or directories containing oteltest scripts",
    )

    args = parser.parse_args()

    if args.verbose:
        logging_level = logging.DEBUG
    else:
        logging_level = logging.INFO

    logging.basicConfig(
        level=logging_level,
        format="> %(message)s",
    )
    logger = logging.getLogger("oteltest")

    run(args.script_paths, args.venv_parent_dir, args.json_dir, logger)


if __name__ == "__main__":
    main()



"""
Causes a ValueError to be raised, caught, and logged. The exception arises because we are trying to log an object.
"""


def main():
    import logging
    import time

    logging.basicConfig(level=logging.NOTSET)
    logging.getLogger().addHandler(create_logging_handler())

    logger = logging.getLogger(__name__)

    try:
        raise ValueError("Err")
    except ValueError as e:
        # log the raw exception object
        logger.error(e)
        # this causes a ValueError to be raised, caught, and logged

    for _ in range(4):
        # demonstrate that the exception was handled and that we can still perform an operation
        time.sleep(1)
        print("x")


def create_logging_handler():
    from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
    from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
    from opentelemetry.sdk._logs.export import BatchLogRecordProcessor

    logger_provider = LoggerProvider()
    logger_provider.add_log_record_processor(
        BatchLogRecordProcessor(OTLPLogExporter(insecure=True))
    )
    return LoggingHandler(logger_provider=logger_provider)


if __name__ == "__main__":
    main()


class MyOtelTest:

    def environment_variables(self):
        return {}

    def requirements(self):
        return ("opentelemetry-exporter-otlp==1.26.0",)

    def wrapper_command(self):
        return ""

    def on_start(self):
        pass

    def on_stop(self, tel, stdout: str, stderr: str, returncode: int) -> None:
        pass

    def is_http(self) -> bool:
        return False
